# Concept: Include Guards 

Include guards are a preprocessor technique used to prevent multiple inclusions of the same header file in a single compilation unit. They are essential for avoiding duplicate definitions and potential compilation errors.

## Example 1: Basic Usage

### Explanation

Here's a basic example of how to use include guards:

```cpp
// myheader.h
#ifndef MYHEADER_H
#define MYHEADER_H

// Header content goes here
struct MyStruct {
    int x;
    int y;
};

#endif // MYHEADER_H
```

- The `#ifndef MYHEADER_H` checks if `MYHEADER_H` is not defined.
- If it's not defined, we define it with `#define MYHEADER_H`.
- The header content is placed between these guards.
- The `#endif` closes the conditional block.

## Example 2: Multiple Inclusions

### Explanation

Let's see how include guards prevent multiple inclusions:

```cpp
// main.cpp
#include "myheader.h"
#include "myheader.h" // This inclusion will be ignored

int main() {
    MyStruct s;
    s.x = 10;
    s.y = 20;
    return 0;
}
```

- The first `#include "myheader.h"` processes the header normally.
- The second inclusion is effectively ignored because `MYHEADER_H` is already defined.
- This prevents duplicate definitions of `MyStruct`.

## Alternative: #pragma once

An alternative to traditional include guards is the `#pragma once` directive:

```cpp
// myheader.h
#pragma once

struct MyStruct {
    int x;
    int y;
};
```

- `#pragma once` is widely supported by modern compilers.
- It's shorter and less error-prone than traditional include guards.
- However, it's not part of the C++ standard, so it may not work on all compilers.

## Summary

Include guards are crucial for preventing multiple inclusions of header files, which can lead to compilation errors due to duplicate definitions. They work by defining a unique macro for each header file and checking if that macro is already defined before processing the file contents. While traditional include guards using `#ifndef`, `#define`, and `#endif` are standard and widely supported, the `#pragma once` directive offers a simpler alternative in many modern compilers. Proper use of include guards is essential for writing robust and maintainable C++ code.

## Citations:
[1] https://acodersjourney.com/top-10-c-header-file-mistakes-and-how-to-fix-them/
[2] https://64.github.io/cpp-faq/include-guards-pragma-once/
[3] https://upcoder.com/20/cc-include-guidelines/
[4] https://www.cs.kent.edu/~nmadi/CS2/Notes/include_guard.html
[5] https://en.wikipedia.org/wiki/Include_guard
[6] https://www.reddit.com/r/C_Programming/comments/wq9u5h/include_guards/
[7] https://computingonplains.wordpress.com/2023/12/05/include-guard-vs-pragma-once-in-c-and-c/
[8] https://stackoverflow.com/questions/27810115/what-exactly-do-c-include-guards-do

## Related
- How do include guards prevent circular references
- What are the differences between include guards and #pragma once
- Can include guards affect build times in large projects
- Are there any performance benefits to using #pragma once over include guards
- How do include guards work in multi-file projects
